<?php

class Scheduled_Woocommerce_Subscription_Status {

	/**
	 * Initialize Hooks.
	 *
	 */
	public function run() {
		/**
		 *  Hook: apply_filters( 'wcs_subscription_statuses', $subscription_statuses())
		 *  in file `woocommerce-subscriptions/woocommerce-subscriptions.php`
		 */
		add_filter( 'woocommerce_subscriptions_registered_statuses', array( $this, 'register_new_post_status' ), 100, 1 );

		/**
		 *  Hook: apply_filters( 'wcs_subscription_statuses', $subscription_statuses )
		 *  in file `woocommerce-subscriptions/wcs-functions.php`
		 */
		add_filter( 'wcs_subscription_statuses', array( $this, 'add_new_subscription_statuses' ), 100, 1 );

		/**
		 *  Hook: apply_filters( 'woocommerce_can_subscription_be_updated_to', $can_be_updated, $new_status, $subscription )
		 *  in file `woocommerce-subscriptions/includes/class-wc-subscription.php`
		 */
		add_filter( 'woocommerce_can_subscription_be_updated_to', array( $this, 'extends_can_be_updated' ), 100, 3 );
		/**
		 * Alternative Hooks available for the above Hook :
		 * apply_filters( 'woocommerce_can_subscription_be_updated_to_' . $new_status, $can_be_updated, $subscription );
		 */

		/**
		 *  Hook: do_action( 'woocommerce_subscription_status_updated', $subscription, $new_status, $old_status )
		 *  in file `woocommerce-subscriptions/includes/class-wc-subscription.php`
		 */
		add_action( 'woocommerce_subscription_status_updated', array( $this, 'extends_update_status' ), 100, 3 );
		/**
		 * Alternative Hooks available for the above Hook :
		 * do_action( 'woocommerce_subscription_status_' . $new_status, $subscription );
		 * do_action( 'woocommerce_subscription_status_' . $old_status . '_to_' . $new_status, $subscription );
		 * do_action( 'woocommerce_subscription_status_updated', $subscription, $new_status, $old_status );
		 * do_action( 'woocommerce_subscription_status_changed', $subscription_id, $old_status, $new_status );
		 */

		 add_action( 'wscsd_start_date_changed', array( $this, 'wscsd_extends_is_activated' ), 100, 2 );

		/**
		 *  Hook: apply_filters( 'woocommerce_can_subscription_be_updated_to_' . $new_status, $can_be_updated, $subscription )
		 *  in file `woocommerce-subscriptions/includes/class-wc-subscription.php`
		 */
		add_filter( 'woocommerce_can_subscription_be_updated_to_active', array( $this, 'enable_active_in_new_statuses' ), 100, 2 );
		add_filter( 'woocommerce_can_subscription_be_updated_to_on-hold', array( $this, 'enable_on_hold_in_new_statuses' ), 100, 2 );

		/**
		 *  Hook: apply_filters( 'woocommerce_subscription_bulk_actions', $bulk_actions )
		 *  in file `woocommerce-subscriptions/includes/class-wc-subscription.php`
		 */
		add_filter( 'woocommerce_subscription_bulk_actions', array( $this, 'add_new_status_bulk_actions' ), 100, 1 );

		/**
		 *  Following is a WordPress core hook. You will find it's woocommerce-subscription implementation
		 *  in file `includes/admin/class-wcs-admin-post-types.php`
		 */
		add_action( 'load-edit.php', array( $this, 'parse_bulk_actions' ) );
	}

	/**
	 * Registered new status by adding `Scheduled` to $registered_statuses array.
	 *
	 * @param array $registered_statuses Registered Statuses array.
	 * @return array $registered_statuses with the new status added to it.

	 */
	public function register_new_post_status( $registered_statuses ) {
		// translators: count
		$registered_statuses['wc-scheduled'] = _nx_noop( 'Scheduled <span class="count">( %s )</span>', 'Scheduled <span class="count">( %s )</span>', 'post status label including post count', 'wscsd' );
		return $registered_statuses;
	}

	/**
	 * Add new status `Scheduled` to $subscription_statuses array.
	 * 
	 * @param array $subscription_statuses current subscription statuses array.
	 * @return array $subscription_statuses with the new status added to it.
	 */
	public function add_new_subscription_statuses( $subscription_statuses ) {
		$subscription_statuses['wc-scheduled'] = _x( 'Scheduled', 'Subscription status', 'wscsd' );
		return $subscription_statuses;
	}

	/**
	 *Extends can_be_updated_to( $status ) functions of Woocommerce Subscription plugin.
	 * 
	 * @param boolean $can_be_updated default value if the current subscription can be updated to new status or not.
	 * @param string $new_status New status to which current subscription it is to be updated.
	 * @param object $subscription current subscription object.
	 * @return boolean $can_be_updated If the current subscription can be updated to new status or not.
	 */
	public function extends_can_be_updated( $can_be_updated, $new_status, $subscription ) {
		if ( 'scheduled' == $new_status ) {
			if ( $subscription->payment_method_supports( 'subscription_suspension' ) && $subscription->has_status( array( 'active', 'pending', 'on-hold' ) ) ) {
				$can_be_updated = true;
			} else {
				$can_be_updated = false;
			}
		}
		return $can_be_updated;
	}

	 //set the scheduled status on subscription with a custom start, which status was changed to active
	public function  wscsd_extends_is_activated ( $subscription_id, $date ) {
		$subscription = new WC_Subscription( $subscription_id );

		//$subscription->update_status( 'on-hold', __( 'Payment completed on order after subscription was cancelled.', 'woocommerce-subscriptions' ) );
		if ( metadata_exists( 'post', $subscription->get_id (), '_wscsd_delayed_start_date' ) ) {
			$scheduled_start = get_post_meta( $subscription->get_id (), '_wscsd_delayed_start_date', true );
			 //$next_payment = get_post_meta( $subscription->get_id (), '_schedule_next_payment', true );
			//$subscription->save();
			if ( ! empty( $scheduled_start ) ) {
				$today = gmdate( 'Y-m-d H:i:s' );

				if ( $today < gmdate( 'Y-m-d H:i:s', strtotime( $scheduled_start ) ) ) {
					//$subscription->set_status( 'wc-scheduled' );
					//$subscription->save();
					$subscription->update_status( 'scheduled' );
					//$subscription->update_dates( array( 'next_payment' => '2021-04-20 20:57:57' ) );
					//update_post_meta( $subscription->get_id (), '_schedule_next_payment', $next_payment );
					//make user inactive
					//wcs_maybe_make_user_inactive( $subscription->customer_user );
				}
			}
		}
	}

	/**
	 * Enable `Active` status in the status change dropdown of the subcription with this new status.
	 * 
	 * @param boolean $can_be_updated default value if the current subscription can be updated to new status or not.
	 * @param object $subscription current subscription object.
	 * @return boolean $can_be_updated If the current Subscription can be updated to new status or not.
	 * This function replaces the default code with the new one.
	 * This code will also activate `reactivate` link in the list page for the subscription with `Scheduled` status
	 *
	 */
	public function enable_active_in_new_statuses( $can_be_updated, $subscription ) {
		if ( $subscription->payment_method_supports( 'subscription_reactivation' ) && $subscription->has_status( array( 'on-hold', 'scheduled' ) ) ) {
			$can_be_updated = true;
		} elseif ( $subscription->has_status( 'pending' ) ) {
			$can_be_updated = true;
		} else {
			$can_be_updated = false;
		}
		return $can_be_updated;
	}

	/**
	 * Enable `On Hold` status in the status change dropdown of the subcription with this new status.
	 * 
	 * @param boolean $can_be_updated default value if the current subscription can be updated to new status or not.
	 * @param object $subscription current subscription object.
	 * @return boolean $can_be_updated If the current subscription can be updated to new status or not.
	 *
	 * This function replaces the default code with the new one.
	 * This code will also activate `suspend` link in the list page for the subscription with `Scheduled` status
	 */
	public function enable_on_hold_in_new_statuses( $can_be_updated, $subscription ) {
		if ( $subscription->payment_method_supports( 'subscription_suspension' ) && $subscription->has_status( array( 'active', 'pending', 'scheduled' ) ) ) {
			$can_be_updated = true;
		} else {
			$can_be_updated = false;
		}
		return $can_be_updated;
	}

	/**
	 * Actions to be performed while the status is updated should be handled here
	 * 
	 * @param object $subscription current subscription object.
	 * @param string $new_status New status to which current subscription it is to be updated.
	 * @param string $old_status Current status of current subscription.
	 * @return boolean $can_be_updated If the current subscription can be updated to new status or not.
	 */
	public function extends_update_status( $subscription, $new_status, $old_status ) {
		if ( 'scheduled' == $new_status ) {
			//$subscription->update_suspension_count( $subscription->suspension_count + 1 );
			wcs_maybe_make_user_inactive( $subscription->customer_user );

		}
	}

	/**
	 * Add the new status on the bulk actions drop down of the link
	 * 
	 * @param array $bulk_actions current bulk action array.
	 * @return array $bulk_actions with the new status added to it.
	 */
	public function add_new_status_bulk_actions( $bulk_actions ) {
		$bulk_actions['scheduled'] = _x( 'Mark Scheduled', 'an action on a subscription', 'wscsd' );
		return $bulk_actions;
	}

	/**
	 * Deals with bulk actions. The style is similar to what WooCommerce is doing. Extensions will have to define their
	 * own logic by copying the concept behind this method.
	 *
	 */
	public function parse_bulk_actions() {

		// We only want to deal with shop_subscriptions. In case any other CPTs have an 'active' action
		if ( !isset( $_REQUEST['post_type'] ) || 'shop_subscription' !== $_REQUEST['post_type'] || !isset( $_REQUEST['post'] ) ) {
			return;
		}

		$action = '';

		if ( isset( $_REQUEST['action'] ) && -1 != $_REQUEST['action'] ) {
			$action = sanitize_text_field( $_REQUEST['action'] );
		} elseif ( isset( $_REQUEST['action2'] ) && -1 != $_REQUEST['action2'] ) {
			$action = sanitize_text_field( $_REQUEST['action2'] );
		}

		switch ( $action ) {
			case 'active':
			case 'on-hold':
			case 'cancelled':
			case 'scheduled':
				$new_status = $action;
				break;
			default:
				return;
		}

		$report_action = 'marked_' . $new_status;

		$changed = 0;

		$subscription_ids = array_map( 'absint', ( array ) $_REQUEST['post'] );

		$sendback_args = array( 
			'post_type' => 'shop_subscription', 
			$report_action => true, 
			'ids' => join( ', ', $subscription_ids ), 
			'error_count' => 0, 
		 );

		foreach ( $subscription_ids as $subscription_id ) {
			$subscription = wcs_get_subscription( $subscription_id );
			$order_note = _x( 'Subscription status changed by bulk edit:', 'Used in order note. Reason why status changed.', 'woocommerce-subscriptions' );

			try {
				if ( 'cancelled' == $action ) {
					$subscription->cancel_order( $order_note );
				} else {
					$subscription->update_status( $new_status, $order_note, true );
				}

				// Fire the action hooks
				switch ( $action ) {
					case 'active':
					case 'on-hold':
					case 'cancelled':
					case 'scheduled':
					case 'trash':
						do_action( 'woocommerce_admin_changed_subscription_to_' . $action, $subscription_id );
						break;
				}

				$changed++;
			} catch ( Exception $e ) {
				$sendback_args['error'] = urlencode( $e->getMessage() );
				$sendback_args['error_count']++;
			}
		}

		$sendback_args['changed'] = $changed;
		$sendback = add_query_arg( $sendback_args, wp_get_referer() ? wp_get_referer() : '' );
		wp_safe_redirect( esc_url_raw( $sendback ) );

		exit();
	}
}
