<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

add_filter( 'woocommerce_subscriptions_recurring_cart_key', 'wscsd_add_custom_cart_key_meta', 10, 2 );

function wscsd_add_custom_cart_key_meta( $cart_key, $cart_item ) {
	//prevent the grouping of subscriptions when they have different start dates by adding the start date to the $cart_key
	if ( array_key_exists( 'wscsd_start_date', $cart_item ) ) {
		$start_date_key = $cart_item['wscsd_start_date'];

		if ( ! empty( $start_date_key ) ) {
			$cart_key .= $start_date_key;
		}
	}
	return $cart_key;
}

add_filter( 'wcs_recurring_cart_next_payment_date', 'wscsd_change_next_date', 100, 3 );
function wscsd_change_next_date( $date, $recurring_cart, $product ) {
	$customer_selected_start_date = '';
	foreach ( $recurring_cart->get_cart() as $cart_item_key => $cart_item ) {
		//if ( ( '' == $customer_selected_start_date ) && ( array_key_exists( 'wscsd_start_date', $cart_item ) ) && ( !empty( $date ) ) ) {
		if ( ( '' == $customer_selected_start_date ) && ( array_key_exists( 'wscsd_start_date', $cart_item ) ) ) {
			$site_time_offset = (int) ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS );
			$customer_selected_start_time = strtotime( $cart_item['wscsd_start_date'] ) + 3600;
			$customer_selected_start_time -= $site_time_offset;
			$customer_selected_start_date = gmdate( 'Y-m-d H:i:s', $customer_selected_start_time );
			$old_start_date = gmdate( 'Y-m-d', strtotime( $recurring_cart->start_date ) );
			$date = wscsd_get_first_renewal_payment_date( $product, $customer_selected_start_date, $old_start_date );
			$today = gmdate( 'Y-m-d');

			$delay_type = get_post_meta( $cart_item['data']->get_id(), '_wscsd_delay_type', true );
			if ( 'product_default' == $delay_type ) {
				$delay_payment = get_post_meta( wp_get_post_parent_id( $cart_item['data']->get_id() ), '_wscsd_delay_payment', true );
				//$delay_payment = true;
			} else {
				$delay_payment = get_post_meta( $cart_item['data']->get_id(), '_wscsd_delay_payment', true );
			}

			//synchronized payment
			$prorate_synced_payments = ( 'no' == get_option( WC_Subscriptions_Admin::$option_prefix . '_sync_payments' ) ? 'none' : get_option( WC_Subscriptions_Admin::$option_prefix . '_prorate_synced_payments', 'none' ) );
			if ( ( true == $delay_payment ) && ( 0 == WC_Subscriptions_Product::get_trial_length( $product ) ) ) {
				//check if payment needs to be delayed
				if ( ( ! WC_Subscriptions_Product::get_trial_length( $product ) ) || ( 0 == WC_Subscriptions_Product::get_trial_length( $product ) ) ) {
					if ( ( 'no' == $prorate_synced_payments ) && ( 0 != $product->subscription_payment_sync_date ) ) {
						//if synchronised payments
						$date = wscsd_get_first_renewal_payment_date( $product, $date, $old_start_date );
					} else {
						$date = $customer_selected_start_date;
					}
					
				} else {
					$trial_length = WC_Subscriptions_Product::get_trial_length( $product );
					$trial_period = WC_Subscriptions_Product::get_trial_period( $product );
					$date = strtotime( $customer_selected_start_date . '+' . $trial_length . ' ' . $trial_period );
				}
			} else {
				$date = wscsd_get_first_renewal_payment_date( $product, $customer_selected_start_date, $old_start_date );
				if ( ( gmdate( 'Y-m-d', strtotime( $date ) ) == gmdate( 'Y-m-d', strtotime( $customer_selected_start_date ) ) || ( gmdate( 'Y-m-d', strtotime( $date ) ) == gmdate( 'Y-m-d', strtotime( $cart_item['wscsd_start_date'] ) ) ) ) ) {
				
					if ( 'recurring' == $prorate_synced_payments ) {
						//check if the amount if charged upfront 'recurring'
						$synchronized_date = gmdate( 'Y-m-d  H:i:s', strtotime( $date . '+1 day' ) );
						//CHECK MIGHT NEED TO DO -1DAY
						$date = wscsd_get_first_renewal_payment_date( $product, $synchronized_date, $old_start_date );
					} /*elseif ( ( 'yes' == $prorate_synced_payments ) || ( ( 'virtual' == $prorate_synced_payments ) && ( $product->is_virtual() ) ) ) {
						//add_filter( 'woocommerce_subscriptions_synced_first_payment_date', function( $first_payment, $product, $type, $from_date, $from_date_param ) use ( $date ) { return $date; }  );
					}*/
				}
			}
			if ( ( $date <= $today ) || ( $today == gmdate( 'Y-m-d', $customer_selected_start_time ))){
				$date = wscsd_get_first_renewal_payment_date( $product, $today, $old_start_date );
			}
			break; 
		}
	}
			
	return $date;
}

function wscsd_get_first_renewal_payment_date( $product, $start_date, $old_start_date ) {

	$next_payment_date = WC_Subscriptions_Product::get_first_renewal_payment_date( $product, $start_date );

	if ( ( 0 == $next_payment_date ) || ( '' == $next_payment_date ) ) {
		$billing_interval = WC_Subscriptions_Product::get_interval( $product );
		$billing_length   = WC_Subscriptions_Product::get_length( $product );
		$trial_length     = WC_Subscriptions_Product::get_trial_length( $product );
		$from_date = $start_date;
		if ( empty( $from_date ) ) {
			$from_date = gmdate( 'Y-m-d H:i:s' );
		}

		// If the subscription has a free trial period, the first renewal payment date is the same as the expiration of the free trial
		if ( $trial_length > 0 ) {

			$first_renewal_timestamp = wcs_date_to_time( WC_Subscriptions_Product::get_trial_expiration_date( $product, $from_date ) );

		} else {

			$site_time_offset = (int) ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS );

			// As wcs_add_time() calls wcs_add_months() which checks for last day of month, pass the site time
			$first_renewal_timestamp = wcs_add_time( $billing_interval, WC_Subscriptions_Product::get_period( $product ), wcs_date_to_time( $from_date ) + $site_time_offset );

			/*if ( 'site' !== $timezone ) {
				$first_renewal_timestamp -= $site_time_offset;
			}*/
		}

		$old_end_date = WC_Subscriptions_Product::get_expiration_date( $product, $old_start_date );
		$diff_payment_dates = ( strtotime( $start_date ) - strtotime( $old_start_date ) );
		$new_end_date = strtotime ( $old_end_date ) + $diff_payment_dates;
		//$end_date = gmdate( 'Y-m-d H:i:s', $new_end_date );
		//$next_payment_date = gmdate( 'Y-m-d H:i:s', $first_renewal_timestamp );
		if ( $new_end_date > $first_renewal_timestamp ) {
			$next_payment_date = gmdate( 'Y-m-d H:i:s', $first_renewal_timestamp );
		}

	}
	return $next_payment_date;

}

add_filter( 'wcs_recurring_cart_end_date', 'wscsd_change_end_date', 100, 3 );
function wscsd_change_end_date( $date, $recurring_cart, $product ) {
	if ( !empty( $date ) ) {
		$customer_selected_start_date = '';
		foreach ( $recurring_cart->get_cart() as $cart_item_key => $cart_item ) {
			if ( ( ''== $customer_selected_start_date ) && ( array_key_exists( 'wscsd_start_date', $cart_item ) ) ) {
				$customer_selected_start_date = gmdate( 'Y-m-d', strtotime( $cart_item['wscsd_start_date'] ) );
				$old_start_date = gmdate( 'Y-m-d', strtotime( $recurring_cart->start_date ) );
				$old_end_date = gmdate( 'Y-m-d', strtotime( $date ) );
				$diff_payment_dates = ( strtotime( $customer_selected_start_date ) - strtotime( $old_start_date ) );
				$new_end_date = strtotime ( $old_end_date ) + $diff_payment_dates;
				$date = gmdate( 'Y-m-d H:i:s', $new_end_date );
				break; 
			}
		}
	}
	
			
	return $date ;
}
add_filter( 'wcs_recurring_cart_trial_end_date', 'wscsd_change_trial_end_date', 100, 3 );
function wscsd_change_trial_end_date( $date, $recurring_cart, $product ) {
	if ( !empty( $date ) ) {
		$customer_selected_start_date = '';
		foreach ( $recurring_cart->get_cart() as $cart_item_key => $cart_item ) {
			if ( ( ''== $customer_selected_start_date ) && ( array_key_exists( 'wscsd_start_date', $cart_item ) ) ) {
				$site_time_offset = (int) ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS );
				$customer_selected_start_date = gmdate( 'Y-m-d', strtotime( $cart_item['wscsd_start_date'] ) );
				$old_start_date = gmdate( 'Y-m-d', strtotime( $recurring_cart->start_date ) );
				$old_trial_end_date = gmdate( 'Y-m-d', strtotime( $date ) );
				$diff_payment_dates = ( strtotime( $customer_selected_start_date ) - strtotime( $old_start_date ) );
				$new_trial_end_date = strtotime ( $old_trial_end_date ) + $diff_payment_dates - $site_time_offset;
				//$new_trial_end_date = strtotime ( $old_trial_end_date ) + $diff_payment_dates;
				$date = gmdate( 'Y-m-d H:i:s', $new_trial_end_date );
				break; 
			}
		}
	}
			
	return $date ;
}

add_filter( 'woocommerce_get_item_data', 'wscsd_start_date_cart_formats', 10, 2 );
//Better display date with site format in cart
function wscsd_start_date_cart_formats( $item_data, $cart_item ) {
	if ( !empty( $item_data ) ) {
		foreach ( $item_data as $index=>$item_column ) {
			if ( ( array_key_exists( 'key', $item_column ) ) && ( 'Start date' == $item_column['key'] ) ) {
				$old_date = $item_data[$index]['value'];
				$formated_date = date_i18n( wc_date_format(), strtotime( $old_date ) );
				$item_data[$index]['value'] = $formated_date;
			}

		}
		
	}

	return $item_data;
}

// Sometimes, even if the order total is $0, the cart still needs payment
		add_filter( 'woocommerce_cart_needs_payment', 'wscsd_cart_needs_payment', 100, 2 );

function wscsd_cart_needs_payment( $needs_payment, $cart ) {
	// Skip checks if needs payment is already set or cart total not 0.
	if ( false !== $needs_payment || 0 != $cart->total ) {
		return $needs_payment;
	}
	/*// Skip checks if cart has no subscriptions.
	if ( ! self::cart_contains_subscription() ) {
		return $needs_payment;
	}*/

	// Skip checks if cart contains subscription switches or automatic payments are disabled.
	if ( false !== wcs_cart_contains_switches( 'any' ) || wcs_is_manual_renewal_required() ) {
		return $needs_payment;
	}
	$recurring_total                  = 0;
	$is_one_period                    = true;
	$contains_delayed_subscription    = false;
	$contains_delayed_subscription = wscsd_contain_delayed_subscription( $cart );
	
	if ( $contains_delayed_subscription ) {
		if ( ! empty( WC()->cart->recurring_carts ) ) {

			$zero_initial_payment_requires_payment = true;
			//Check if allow $0 checkout without payment method
			if ( class_exists( 'WCS_Zero_Initial_Payment_Checkout_Manager' ) ) {
				$zero_initial_payment_requires_payment = WCS_Zero_Initial_Payment_Checkout_Manager::zero_initial_checkout_requires_payment();
			}
			
			foreach ( WC()->cart->recurring_carts as $recurring_cart ) {
				$recurring_total                 += $recurring_cart->total;
				$subscription_length              = wcs_cart_pluck( $recurring_cart, 'subscription_length' );
				if ( 0 == $subscription_length || wcs_cart_pluck( $recurring_cart, 'subscription_period_interval' ) != $subscription_length ) {
					$is_one_period = false;
				}
			}
		}

		//check if contains delayed subs
		if ( ( $recurring_total > 0 ) && ( $is_one_period ) && ( $zero_initial_payment_requires_payment ) ) {
			$needs_payment = true;
		}
	}
	
	return $needs_payment;
}

function wscsd_contain_delayed_subscription( $cart = null ) {
	$cart            = ( empty( $cart ) && isset( WC()->cart ) ) ? WC()->cart : $cart;
	$contains_delayed_subscription = false;

	if ( ! empty( $cart ) ) {

		foreach ( WC()->cart->get_cart() as $cart_item ) {
			if ( array_key_exists( 'wscsd_start_date', $cart_item ) ) {
				$contains_delayed_subscription = $cart_item;
				break;
			}
		}
	}

	return $contains_delayed_subscription;
}
/*
add_filter( 'woocommerce_subscriptions_cart_get_price', 'wscsd_pay_later_price', 100, 2 );
function wscsd_pay_later_price( $price, $product ) {

	if ( WC_Subscriptions_Product::is_subscription( $product ) ) {
		$calculation_type = WC_Subscriptions_Cart::get_calculation_type();

			// For original calculations, we need the items price to account for delayed payments
			if ( 'none' == $calculation_type ) {

				$price = 0;
			}  
 
	}

	return $price;
}

//apply_filters( 'woocommerce_subscriptions_cart_shipping_up_front', $charge_shipping_up_front );


function cart_contains_delayed_payment() {

		$cart_contains_free_trial = false;

		if ( WC_Subscriptions_Cart::cart_contains_subscription() ) {
			foreach ( WC()->cart->cart_contents as $cart_item ) {
				if ( WC_Subscriptions_Product::get_trial_length( $cart_item['data'] ) > 0 ) {
					$cart_contains_free_trial = true;
					break;
				}
			}
		}

		return $cart_contains_free_trial;
	}

/*function wscsd_add_cart_first_renewal_payment_date( $order_total_html, $cart ) {

	if ( 0 !== $cart->next_payment_date ) {
		$first_renewal_date = date_i18n( wc_date_format(), wcs_date_to_time( get_date_from_gmt( $cart->next_payment_date ) ) );
		// translators: placeholder is a date
		//$order_total_html  .= '<div class="first-payment-date"><small>' . sprintf( __( 'First payment: %s', 'woocommerce-subscriptions' ), $first_renewal_date ) . '</small></div>';
	}

	return $order_total_html;
}
add_filter( 'wcs_cart_totals_order_total_html', 'wscsd_add_cart_first_renewal_payment_date', 4, 2 );


//apply_filters( 'wcs_cart_totals_order_total_html', 'wscsd_change_first_renewal_text', 1000, 2 );

function wscsd_change_first_renewal_text ( $order_total_html, $cart ){
	$order_total_html_2 = preg_replace( 'renewal', 'payment', $order_total_html );

	return $order_total_html_2;

}

add_filter( 'wcs_recurring_cart_next_payment_date', 'wscsd_change_next_date_2', 1000, 3 );

function wscsd_change_next_date_2( $date, $recurring_cart, $product ){
	$customer_selected_start_date = '';
	foreach ( $recurring_cart->get_cart() as $cart_item_key => $cart_item ) {
		if ( ( ''== $customer_selected_start_date ) && ( array_key_exists( 'wscsd_start_date', $cart_item ) ) && ( !empty( $date ) ) ) {
			$date = gmdate( 'Y-m-d H:i:s', strtotime( $cart_item['wscsd_start_date'] ) );
		}
	}
			
	return $date ;

}*/


// Remove any subscriptions with a future start date from the initial shipping packages
		add_filter( 'woocommerce_cart_shipping_packages', 'wscsd_set_cart_shipping_packages', -10, 1 );
/**
 * Filters the cart contents to remove any subscriptions with free trials (or synchronised to a date in the future)
 * to make sure no shipping amount is calculated for them.
 *
 * @since 2.0
 */
function wscsd_set_cart_shipping_packages( $packages ) {
	if ( ! WC_Subscriptions_Cart::cart_contains_subscription() ) {
		return $packages;
	}

		
	$sub_cart = new WC_Subscriptions_Cart();
	$cart_calculation = new ReflectionProperty('WC_Subscriptions_Cart', 'calculation_type');
	$cart_calculation->setAccessible(true);
	if ( 'none' == $cart_calculation->getValue( $sub_cart ) ) {
		foreach ( $packages as $index => $package ) {
			foreach ( $package['contents'] as $cart_item_key => $cart_item ) {
				if ( array_key_exists( 'wscsd_start_date', $cart_item ) ) {
					//check for custom start date
					if ( gmdate( 'Y-m-d') < $cart_item['wscsd_start_date'] ) {
						//check if start date is in the future
						$delay_type = get_post_meta( $cart_item['data']->get_id(), '_wscsd_delay_type', true );
						if ( 'product_default' == $delay_type ) {
							$delay_payment = get_post_meta( wp_get_post_parent_id( $cart_item['data']->get_id() ), '_wscsd_delay_payment', true );
							$delay_payment = true;
						} else {
							$delay_payment = get_post_meta( $cart_item['data']->get_id(), '_wscsd_delay_payment', true );
						}
						if ( ( true == $delay_payment ) && ( 0 == WC_Subscriptions_Product::get_trial_length( $cart_item['data'] ) ) ) {
							//check payment need to be delayed
							unset( $packages[ $index ]['contents'][ $cart_item_key ] );
						}
					}
				}	
			}

			if ( empty( $packages[ $index ]['contents'] ) ) {
				unset( $packages[ $index ] );
			}
		}
	} 
	return $packages;

}
		//add_action( 'woocommerce_subscriptions_cart_get_price', 'WSCSD_Subscriptions_Cart::wscsd_start_date_price_filter', 10, 2 );
/*class WSCSD_Subscriptions_Cart extends WC_Subscriptions_Cart {

	public static function wscsd_start_date_price_filter( $price, $product ){

$sub_cart = new WC_Subscriptions_Cart();
$cart_calculation = new ReflectionProperty('WC_Subscriptions_Cart', 'calculation_type');
$cart_calculation->setAccessible(true);
		if ( 'none' == $cart_calculation->getValue( $sub_cart ) ) {
			foreach ( WC()->cart->get_cart() as $cart_item ) {
				if ( $product == $cart_item['data'] ) {
					if ( ( array_key_exists( 'wscsd_start_date', $cart_item ) ) && ( array_key_exists( 'wscsd_delay_payment', $cart_item ) ) ) {
						if ( 0 == $cart_item['wscsd_delay_payment'] ) {
							if ( gmdate( 'Y-m-d') < $cart_item['wscsd_start_date'] ) {
								$sign_up_fee = WC_Subscriptions_Product::get_sign_up_fee( $product );
								// Extra check to make sure that the sign up fee is numeric before using it
								$sign_up_fee = is_numeric( $sign_up_fee ) ? (float) $sign_up_fee : 0;
								$price = $sign_up_fee;
								$cart_item['wscsd_delay_payment'] = 1;
								break;
							}
							
						}
					}

				}
			}
		}
		return $price;
	
	}

}*/
	
	
//add_filter( 'woocommerce_calculated_total', 'wscsd_payment_delay',100, 2 );
/*function wscsd_payment_delay($total, $cart){

	$sub_cart = new WC_Subscriptions_Cart();
$cart_calculation = new ReflectionProperty('WC_Subscriptions_Cart', 'calculation_type');
$cart_calculation->setAccessible(true);
		if ( 'none' == $cart_calculation->getValue( $sub_cart ) ) {
			foreach ( WC()->cart->get_cart() as $cart_item ) {
				if ( array_key_exists( 'wscsd_start_date', $cart_item ) ) {
						if ( gmdate( 'Y-m-d') < $cart_item['wscsd_start_date'] ) {
							$price = WC_Subscriptions_Product::get_price( $cart_item['data'] );
							$trial_length = WC_Subscriptions_Product::get_trial_length( $cart_item['data'] );
							$sign_up_fee = WC_Subscriptions_Product::get_sign_up_fee( $cart_item['data'] );
							// Extra check to make sure that the sign up fee is numeric before using it
							$sign_up_fee = is_numeric( $sign_up_fee ) ? (float) $sign_up_fee : 0;
							if ( 0 >= $trial_length ) {
								$total -= $price;
							} 
						}
						
					}

			}
		}
		return $total;
}*/

//Remove the price of subscriptions with delayed payment
add_action( 'woocommerce_before_calculate_totals', 'wscsd_remove_delayed_cost', 10, 1);
function wscsd_remove_delayed_cost( $cart_object ) {

	if ( is_admin() && ! defined( 'DOING_AJAX' ) ) {
		return;
	}

	/*if ( did_action( 'woocommerce_before_calculate_totals' ) >= 3 ) {
		return;
	}*/

	$sub_cart = new WC_Subscriptions_Cart();
	$cart_calculation = new ReflectionProperty('WC_Subscriptions_Cart', 'calculation_type');
	$cart_calculation->setAccessible(true);
	if ( 'none' == $cart_calculation->getValue( $sub_cart ) ) {
		foreach ( WC()->cart->get_cart() as $cart_item ) {
			if ( array_key_exists( 'wscsd_start_date', $cart_item ) ) {
				if ( gmdate( 'Y-m-d') < $cart_item['wscsd_start_date'] ) {
					//check if start date is in the future
					$delay_type = get_post_meta( $cart_item['data']->get_id(), '_wscsd_delay_type', true );
					if ( 'product_default' == $delay_type ) {
						$delay_payment = get_post_meta( wp_get_post_parent_id( $cart_item['data']->get_id() ), '_wscsd_delay_payment', true );
					} else {
						$delay_payment = get_post_meta( $cart_item['data']->get_id(), '_wscsd_delay_payment', true );
					}
					if ( ( true == $delay_payment ) && ( 0 == WC_Subscriptions_Product::get_trial_length( $cart_item['data'] ) ) ) {
						$price = WC_Subscriptions_Product::get_price( $cart_item['data'] );
						$sign_up_fee = WC_Subscriptions_Product::get_sign_up_fee( $cart_item['data'] );
						// Extra check to make sure that the sign up fee is numeric before using it
						$sign_up_fee = is_numeric( $sign_up_fee ) ? (float) $sign_up_fee : 0;
						## Set the price with WooCommerce compatibility ##
						if ( version_compare( WC_VERSION, '3.0', '<' ) ) {
							$cart_item['data']->price = 0; // Before WC 3.0
						} else {
							$cart_item['data']->set_price( 0); // WC 3.0+
						}
					}
				}
			}
		}
	}
}

add_action( 'woocommerce_subscription_cart_after_grouping', 'wscsd_reset_recurring_prices', 1000 );
function wscsd_reset_recurring_prices() {
	$sub_cart = new WC_Subscriptions_Cart();
	$cart_calculation = new ReflectionProperty('WC_Subscriptions_Cart', 'calculation_type');
	$cart_calculation->setAccessible(true);
	foreach ( WC()->cart->get_cart() as $cart_item ) {
		if ( array_key_exists( 'wscsd_start_date', $cart_item ) ) {
			if ( gmdate( 'Y-m-d' ) < $cart_item['wscsd_start_date'] ) {
				//check if start date is in the future
				$delay_type = get_post_meta( $cart_item['data']->get_id(), '_wscsd_delay_type', true );
				if ( 'product_default' == $delay_type ) {
					$delay_payment = get_post_meta( wp_get_post_parent_id( $cart_item['data']->get_id() ), '_wscsd_delay_payment', true );
					$delay_payment = true;
				} else {
					$delay_payment = get_post_meta( $cart_item['data']->get_id(), '_wscsd_delay_payment', true );
				}
				if ( ( true == $delay_payment ) && ( 0 == WC_Subscriptions_Product::get_trial_length( $cart_item['data'] ) ) ) {
					$price = WC_Subscriptions_Product::get_price( $cart_item['data'] );
					$trial_length = WC_Subscriptions_Product::get_trial_length( $cart_item['data'] );
					$sign_up_fee = WC_Subscriptions_Product::get_sign_up_fee( $cart_item['data'] );
					// Extra check to make sure that the sign up fee is numeric before using it
					$sign_up_fee = is_numeric( $sign_up_fee ) ? (float) $sign_up_fee : 0;
					
					## Set the price with WooCommerce compatibility ##
					if ( version_compare( WC_VERSION, '3.0', '<' ) ) {
						$cart_item['data']->price = $price; // Before WC 3.0
					} else {
						$cart_item['data']->set_price( $price); // WC 3.0+
					}
				} 
			}
		}
	}
}
