<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

add_action( 'woocommerce_checkout_create_subscription', 'wscsd_store_start_date', 10, 4 );

function wscsd_store_start_date( $subscription, $posted_data, $order, $recurring_cart ) {
   
	$customer_selected_start_date = '';
	foreach ( $recurring_cart->get_cart() as $cart_item_key => $cart_item ) {
		if ( ( ''== $customer_selected_start_date ) && ( array_key_exists( 'wscsd_start_date', $cart_item ) ) ) {
			$site_time_offset = (int) ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS );
			$customer_selected_start_date = $cart_item['wscsd_start_date'];
			$date = gmdate( 'Y-m-d H:i:s', strtotime ( $customer_selected_start_date ) - $site_time_offset );
			$delayed_payment = array_key_exists( 'wscsd_delay_payment', $cart_item );
			$next_payment_date = gmdate( 'Y-m-d H:i:s', wcs_date_to_time( $recurring_cart->next_payment_date ));
			break; 
		}
	}
	if ( !empty ( $date ) ) {

		add_post_meta( $subscription->get_id (), '_wscsd_delayed_start_date', $date );
		add_post_meta( $subscription->get_id (), '_wscsd_delayed_payment', $delayed_payment );
		add_post_meta( $subscription->get_id (), '_wscsd_next_payment_date', $next_payment_date );
	}
		

}

add_action( 'subscriptions_activated_for_order', 'wscsd_update_start_date', 100, 1 );
//Update the start date of the subscription
function wscsd_update_start_date( $order_id ) {
	 $contains_subscription = wcs_order_contains_subscription( $order_id );

	if ( !$order_id || !$contains_subscription ) {
		return;
	}
	$subscriptions = wcs_get_subscriptions_for_order( $order_id, array( 'order_type' => 'any' ) );
	// We get all related subscriptions for this order
	//foreach ( $subscriptions as $subscription_id => $subscription ) {
	foreach ( $subscriptions as $subscription ) {
	 $subscription_id = $subscription->get_id();
		//if( $subscription_obj->order->id == $order_id ) break; // Stop the loop
		if ( !empty ( get_metadata ( 'post', $subscription_id , '_wscsd_delayed_start_date', true ) ) ) {
			$start_date = get_metadata ( 'post', $subscription_id , '_wscsd_delayed_start_date', true );
			
			/*if ( metadata_exists('post', $subscription_id, '_schedule_start') ) {
				update_post_meta( $subscription_id, '_schedule_start', $start_date );
			} else {
				$subscription->update_meta_data( '_schedule_start', $start_date );
			}*/
			
			
			
			//if expires before next payment date
			$existing_next_payment_date = $subscription->get_date( 'next_payment' );
			$end = $subscription->get_date( 'end' );
			if ( ( $existing_next_payment_date < $start_date ) || ( ( $existing_next_payment_date > $end ) && ( 0 != $end ) ) ) {
				//$subscription_obj->delete_date( 'next_payment' );
				$subscription->update_dates( 
					array( 
						'next_payment' => '',
					)
				);
			} else {
				//Update next payment
				//$subscription = wcs_get_subscription( $subscription_id );
				
				$subscription->update_dates( 
						array( 
							'next_payment' => '',
						)
					);
				$next_payment_date = $subscription->calculate_date( 'next_payment' );

				//check if next payment is before trial end because of timezone difference
				$trial_end = $subscription->get_time( 'trial_end' );
				if ( $next_payment_date < $trial_end ) {
					$subscription->update_dates( 
						array( 
							'next_payment' => $trial_end,
						)
					);
				}

				if ( ( '' == $next_payment_date ) || ( 0 == $next_payment_date ) || ( ( $next_payment_date > $end ) && ( 0 != $end ) ) ) {
					$subscription->update_dates( 
						array( 
							'next_payment' => '',
						)
					);
				} else {
					if ( !( WC_Subscriptions_Synchroniser::subscription_contains_synced_product( $subscription ) ) && ( $existing_next_payment_date < $next_payment_date ) && ( strtotime( $existing_next_payment_date) != strtotime( $start_date ) + 3600 ) ) {
					//check if payment is on start date (delayed)
						$subscription->update_dates( array( 'next_payment' => $next_payment_date ) );
					} else {
						$subscription->update_dates( 
							array( 
								'next_payment' => $existing_next_payment_date,
							)
						);
					}
				}
				
			}
			$subscription->update_dates( 
				array( 
					'start' => $start_date,
				)
			);
			if ( $start_date <= gmdate('Y-m-d') ) {
				$new_payment_date = get_metadata ( 'post', $subscription_id , '_wscsd_next_payment_date', true );
				$subscription->update_dates( 
					array( 
						'next_payment' => $new_payment_date,
					)
				);
			}
			
			do_action ( 'wscsd_start_date_changed', $subscription_id, $start_date );
				
		}
		
	}
}


add_filter( 'woocommerce_subscriptions_thank_you_message', 'wscsd_thank_you_message', 10, 2 );

function wscsd_thank_you_message ( $thank_you_message, $order_id ) {

	if ( wcs_order_contains_subscription( $order_id, 'any' ) ) {
		$scheduled_order = 0;

		//Item count in order
		$order = wc_get_order( $order_id );
		$total_quantity = $order->get_item_count();

		//Check Subscriptions
		$subscriptions                = wcs_get_subscriptions_for_order( $order_id, array( 'order_type' => 'any' ) );
		$subscription_count           = count( $subscriptions );
		$new_message				  = '';
		$my_account_subscriptions_url = get_permalink( wc_get_page_id( 'myaccount' ) );
		$check_start_date = '';

		if ( $total_quantity == $subscription_count ) {
			$scheduled_order = 1;
		}
		
		if ( $subscription_count ) {
			foreach ( $subscriptions as $subscription ) {
				if ( $subscription->has_status( 'scheduled' ) ) {
					$start_date = get_metadata ( 'post', $subscription->get_id(), '_wscsd_delayed_start_date', true );
					$site_time_offset = (int) ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS );
					 $formated_start_date = date_i18n( wc_date_format(), strtotime( $start_date ) + $site_time_offset );
					 // translators: placeholders are opening and closing link tags
					 $new_message = '<p>' . sprintf( _n( 'Your subscription will be activated the %s ', 'Your subscriptions will be activated the', $subscription_count, 'wscsd' ) . $formated_start_date, '</a>' ) . '</p>';

					/*Make sure tall the subcriptions start on the same day
					if ( ( '' != $check_start_date ) && ( $check_start_date != $start_date ) ) {
						$scheduled_order = 0;
					}
					$check_start_date = $start_date;*/

					//Pick up earliest start date
					if ( ( '' == $check_start_date ) || ( $start_date < $check_start_date ) ) {
						$check_start_date = $start_date;
					}
				} else {
					$scheduled_order = 0;
				}
			}
		}
		if ( ! empty( $new_message ) ) {
			 // translators: placeholders are opening and closing link tags
			$new_message .= '<p>' . sprintf( _n( 'View the status of your subscription in %1$syour account%2$s.', 'View the status of your subscriptions in %1$syour account%2$s.', $subscription_count, 'wscsd' ), '<a href="' . $my_account_subscriptions_url . '">', '</a>' ) . '</p>';
			$thank_you_message = $new_message;
		}

		if ( ( 1 == $scheduled_order ) && ( $check_start_date > gmdate( 'Y-m-d H:i:s' ) ) ) {
			if ( $order->get_status() == 'processing' ) {
				$order->update_status( 'scheduled' );
				do_action ( 'wscsd_start_date_order', $order_id, $check_start_date );
			}
		}
		   
	}
	return $thank_you_message;
}



add_action( 'woocommerce_email_after_order_table', 'wscsd_add_sub_info_email', 5, 3 );

/**
 * Adds the subscription information to our order emails.
 *
 * @since 1.5
 */
function wscsd_add_sub_info_email( $order, $is_admin_email, $plaintext = false ) {
	remove_action( 'woocommerce_email_after_order_table', array( 'WC_Subscriptions_Order', 'add_sub_info_email' ), 15, 3 );

	$subscriptions = wcs_get_subscriptions_for_order( $order, array( 'order_type' => 'any' ) );

	if ( ! empty( $subscriptions ) ) {

		$template_base  = plugin_dir_path( __DIR__ ) . 'templates/';
		$template = ( $plaintext ) ? 'emails/plain/subscription-info.php' : 'emails/subscription-info.php';

		wc_get_template(
			$template,
			array(
				'order'          => $order,
				'subscriptions'  => $subscriptions,
				'is_admin_email' => $is_admin_email,
			),
			'',
			$template_base
		);
	}
}
