<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

add_action( 'woocommerce_process_shop_order_meta', 'wscsd_save_custom_start_date', 10, 2 );

function wscsd_save_custom_start_date( $subscription_id ) {
	if ( ( wcs_is_subscription( $subscription ) ) || ( wcs_is_subscription( $subscription_id ) ) ) {
			$subscription = wcs_get_subscription( $subscription_id );
		if ( ( 'scheduled' === get_post_status( $subscription ) || ( $subscription->has_status( 'scheduled' ) ) ) ) {
		//check it's a subscription and it's has the status 'scheduled'
			if ( ! isset( $_POST['_wscsd_nonce'] ) || ! wp_verify_nonce( sanitize_key( $_POST['_wscsd_nonce'] ), 'wcs_custom_subscription_start' ) ) {
				return;
			}
			
			$start_date_timestamp_input = isset( $_POST[ 'wscsd_start_timestamp_utc' ] ) ? wc_clean( $_POST[ 'wscsd_start_timestamp_utc' ] ) : 0;
			$old_start_date = $subscription->get_date( 'start' );

			if ( strtotime( $old_start_date ) != $start_date_timestamp_input ) {
				$new_start_date = gmdate( 'Y-m-d H:i:s', $start_date_timestamp_input );
				$subscription_items = $subscription->get_items();
				if ( 0 < count( $subscription_items ) ) {
					foreach ( $subscription_items as $item_id => $item ) {
						$product = $item->get_product();
					}
				}
				$old_payment_date = wcs_get_subscription( $subscription_id )->get_date( 'next_payment' );
				$old_trial_end_date = wcs_get_subscription( $subscription_id )->get_date( 'trial_end' );
				$old_end_date = wcs_get_subscription( $subscription_id )->get_date( 'end' );
				$site_time_offset = (int) ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS );

				//Next payment
				$next_payment_timestamp_input = isset( $_POST[ 'next_payment_timestamp_utc' ] ) ? wc_clean( $_POST[ 'next_payment_timestamp_utc' ] ) : 0;
				if ( ( strtotime( $old_payment_date ) - $site_time_offset == $start_date_timestamp_input ) || ( empty($_POST[ 'trial_end' ] ) ) ) {
					//recalculate end date based on new start date
					$delayed_payment = get_metadata ( 'post', $subscription_id , '_wscsd_delayed_payment', true );
					if (  1 == $delayed_payment ) {
						//if payment delayed
						//$first_payment_date = $new_start_date;
						$first_payment_date =  gmdate( 'Y-m-d H:i:s', $start_date_timestamp_input + 3600 );
					} else {
						$first_payment_date = WC_Subscriptions_Product::get_first_renewal_payment_date( $product, $new_start_date );	
					} 
				} else {
					//force the end date manually
					$first_payment_date = gmdate( 'Y-m-d H:i:s', $next_payment_timestamp_input );

				}
				//End
				$end_timestamp_input = isset( $_POST[ 'end_timestamp_utc' ] ) ? wc_clean( $_POST[ 'end_timestamp_utc' ] ) : 0;
				if ( ( strtotime( $old_end_date ) == $end_timestamp_input ) || ( empty($_POST[ 'end' ] ) ) ) {
					//recalculate end date based on new start date
					$new_end = WC_Subscriptions_Product::get_expiration_date( $product, $new_start_date );
				} else {
					//force the end date manually
					$new_end = $end_timestamp_input;
				}

				
				
				//trial-end
				$input_trial_end = ( isset( $_POST[ 'trial_end' ] ) ? wc_clean( $_POST[ 'trial_end' ] ) : '' ) . ' ' . ( isset( $_POST[ 'trial_end_hour' ] ) ? wc_clean( $_POST[ 'trial_end_hour' ] ) : '00' ) . ':' . ( isset( $_POST[ 'trial_end_minute' ] ) ? wc_clean( $_POST[ 'trial_end_minute' ] ) : '00' ) . ':00';
				$new_trial_end = '';
				if (!(empty($_POST[ 'trial_end' ] ) ) ) {
					if ( $input_trial_end == $old_trial_end_date ) {
					//recalculate end date based on new start date
					$new_trial_end = WC_Subscriptions_Product::get_trial_expiration_date( $product, $new_start_date );
					} else {
						//force the end date manually
						$new_trial_end = $input_trial_end;
					}
				}

				
				$subscription->update_dates( 
					array( 
						'start' => $new_start_date,
						'next_payment' => $first_payment_date,
						'end' => $new_end,
						'trial_end' => $new_trial_end,
				));
				
				do_action ( 'wscsd_start_date_changed', $subscription_id, $new_start_date );
				if ( '' != $first_payment_date ) {
					do_action ( 'wscsd_set_next_payment_date', $subscription_id, $first_payment_date );
					//as_schedule_single_action( $first_payment_date, 'woocommerce_scheduled_subscription_payment', array( 'subscription_id' => $subscription_id ) );
				}
				if ( ( '' != $new_end ) && ( $old_end_date != $new_end ) ) {
					as_schedule_single_action( $new_end, 'woocommerce_scheduled_subscription_end', array( 'subscription_id' => $subscription_id ) );
				}
				if ( ( '' != $new_trial_end ) && ( $old_trial_end_date != $new_trial_end ) ) {
					as_schedule_single_action( $new_trial_end, 'woocommerce_scheduled_subscription_trial_end', array( 'subscription_id' => $subscription_id ) );
				}
			}
		}
	}
}


//Authorize update of start date from admin

add_action( 'wcs_subscription_schedule_after_billing_schedule', 'wscsd_change_start_date_admin', 10 );

function wscsd_change_start_date_admin ( $the_subscription ) {
	//Add new start date field
	if ( $the_subscription->has_status( 'scheduled' ) ) {
		// check if the subscription is scheduled
		echo '<div id="subscription-wscsd-start-date" class="date-fields">
			<strong>Start Date</strong>
			<input type="hidden" name="wscsd_start_timestamp_utc" id="wscsd_start_timestamp_utc" value="' . esc_attr( $the_subscription->get_time( 'start', 'gmt' ) ) . '"/>';
		echo wp_kses( wcs_date_input( $the_subscription->get_time( 'start', 'gmt' ), array( 'name_attr' => 'wscsd_start' ) ), array( 'input' => array( 'type' => array(), 'class' => array(), 'placeholder' => array(), 'name' => array(), 'id' => array(), 'maxlength' => array(), 'size' => array(), 'value' => array(), 'patten' => array() ), 'div' => array( 'class' => array() ), 'span' => array(), 'br' => array() ) );
			//echo wp_kses( wcs_date_input(0));

		wp_nonce_field( 'wcs_custom_subscription_start', '_wscsd_nonce' );
		echo '</div>';

		
	}

}
